<?php
/**
 * Plugin to add matomo script in public part
 *
 * @author Denis Chenu <denis@sondages.pro>
 * @copyright 2019 Denis Chenu <http://www.sondages.pro>
 * @license GPL v3
 * @version 0.1.0
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */
class matomoPublicTrack extends PluginBase {

    protected $storage = 'DbStorage';

    static protected $description = 'Add matomo script and img in public part.';
    static protected $name = 'matomoPublicTrack';

    /**
    * track if javascript is done
    * @var boolean
    */
    private $registeredTrackingCode;

    /**
    * The currnt controller
    * @var string
    */
    public $sMatomoCustomController = 'default';

    /**
    * The custom parameters for the final url
    * @var array : all url params finally used
    */
    public $aMatomoCustomUrlParams = array(
        'base'=>null,
        'sid'=>null,
        'gid'=>null,
        'qid'=>null,
        'event'=>null,
        'action'=>null,
        'lang' => null,
    );

    /**
    * Matomo plugin optionnal params
    * @var array : all url params finally used
    */
    public $aMatomoPluginParams = array(
        'sid'=>null,
        'srid'=>null,
        'token'=>null,
        'email'=>null,
        'firstname'=>null,
        'lastname' => null,
    );

    /** The settings **/
    protected $settings = array(
        'trackSurveyPages' => array(
            'type'=>'select',
            'options'=>array(
                0=>'No',
                1=>'Yes'
            ),
            'label'=>'Track Survey pages by default.',
            'default'=> 1
        ),
        'matomoURL'=>array(
            'type'=>'string',
            'label'=>'URL to matomo',
            'help' => "Can be relative or absolute",
            'default'=>'/piwik/'
        ),
        'siteID'=>array(
            'type'=>'int',
            'label'=>"Piwik SiteId",
            'help' => "<a href='https://matomo.org/faq/general/faq_19212/' target='_new'>What should you put here?</a>",
            'htmlOptions'=>array(
                'min' => 0,
            ),
            'default'=>1
        ),
        'rewriteURLs'=>array(
            'type'=>'select',
            'options'=>array(
                'tweak'=>'Yes, to a tweaked URL',
                'admin'=>'Yes, to corresponding admin page',
                'public'=>'Yes, to public page with more information (start of survey).',
            ),
            'htmlOptions'=>array(
                'empty' => 'None, usage of real url',
            ),
            'label'=>'Rewrite Survey page URLs to store more information in matomo, tweaked show page as survey with sub page as group … etc… ',
            'default'=>'tweak'
        ),
        //~ 'usePHPApi' => array(
            //~ 'type'=>'select',
            //~ 'options'=>array(
                //~ 0=>'No',
                //~ 1=>'Yes'
            //~ ),
            //~ 'label'=>'Use php API (and not the javascript solution).',
            //~ 'help' => 'See <a href="https://github.com/matomo-org/matomo-php-tracker" target="_blank">matomo-php-tracker repository</a> for more information',
            //~ 'default'=> 0
        //~ ),
        'respectDNT' => array(
            'type'=>'select',
            'options'=>array(
                0=>'No',
                1=>'Yes'
            ),
            'label'=>'Respect Do Not Track.',
            'default'=> 1
        ),
        'matomoUserId' => array(
            'type'=>'string',
            'label'=>'Set matomo user id',
            'help' => "If you set a value here : LimeSurvey use it to construct <a href='https://matomo.org/docs/user-id' target='_blank'>the matomo user id</a>. You can use expression manager, it final user is is empty : no user id is set.",
            'default'=>''
        ),
        'customVariablesNumber' => array(
            'type'=>'int',
            'htmlOptions'=>array(
                'min' => 0,
            ),
            'label'=>'Number of custom variables',
            'help'=>"See <a href='https://matomo.org/docs/custom-variables/' target='_new'>Custom Variables on matomo website</a>, set the max number of available variables. Label is global, value can be updated in survey.",
            'default'=>0
        ),
        'customDimensionsNumber' => array(
            'type'=>'int',
            'htmlOptions'=>array(
                'min' => 0,
            ),
            'label'=>'Number of custom dimensions',
            'help'=>"See <a href='https://matomo.org/docs/custom-dimensions/' target='_new'>Custom Dimensions on matomo website</a>, set the max number of available dimensions.",
            'default'=>0
        ),
        'customVariables' => array(
            'type'=>'info',
            'content'=>'',
        ),
        'customDimensions' => array(
            'type'=>'info',
            'content'=>'',
        ),
    );

    /** Register to events **/
    public function init()
    {
        $this->subscribe('beforeSurveyPage');
        $this->subscribe('beforeControllerAction');
        $this->subscribe('beforeCloseHtml');

        /* To add specific params */
        $this->subscribe('beforeQuestionRender');
        $this->subscribe('afterSurveyComplete');

        /* Params */
        $this->subscribe('beforeToolsMenuRender');

    }

    /** @inheritdoc **/
    public function beforeToolsMenuRender()
    {
        $event = $this->getEvent();
        $surveyId = $event->get('surveyId');
        $aMenuItem = array(
            'label' => $this->gT('Matomo'),
            'iconClass' => 'fa fa-thermometer-half',
            'href' => Yii::app()->createUrl(
                'admin/pluginhelper',
                array(
                    'sa' => 'sidebody',
                    'plugin' => get_class($this),
                    'method' => 'actionSettings',
                    'surveyId' => $surveyId
                )
            ),
        );
        if (class_exists("\LimeSurvey\Menu\MenuItem")) {
            $menuItem = new \LimeSurvey\Menu\MenuItem($aMenuItem);
        } else {
            $menuItem = new \ls\menu\MenuItem($aMenuItem);
        }
        $event->append('menuItems', array($menuItem));
    }

    /**
     * Main function for settings
     * @param int $surveyId Survey id
     *
     * @return string
     */
    public function actionSettings($surveyId)
    {
        $oSurvey=Survey::model()->findByPk($surveyId);
        if(!$oSurvey) {
            throw new CHttpException(404,$this->translate("This survey does not seem to exist."));
        }
        if(!Permission::model()->hasSurveyPermission($surveyId,'surveysettings','update')){
            throw new CHttpException(401);
        }
        if(App()->getRequest()->getPost('save'.get_class($this))) {
            $postedValues = App()->getRequest()->getPost(get_class($this).'settings');
            if(App()->getRequest()->getPost('save'.get_class($this)=='redirect')) {
                Yii::app()->getController()->redirect(Yii::app()->getController()->createUrl('admin/survey',array('sa'=>'view','surveyid'=>$surveyId)));
            }
        }

        $aData['pluginClass']=get_class($this);
        $aData['surveyId']=$surveyId;
        $aData['lang'] = array(
            'Matomo settings for this survey' => gT("Organize question groups/questions"),
        );
        $aData['form'] = array(
            'action' => Yii::app()->createUrl('admin/pluginhelper/sa/sidebody',array('plugin' => get_class($this),'method' => 'actionSaveSettings','surveyId' => $surveyId)),
            'reset' => Yii::app()->createUrl('admin/pluginhelper/sa/sidebody',array('plugin' => get_class($this),'method' => 'actionSettings','surveyId' => $surveyId)),
            'close' => Yii::app()->createUrl('admin/survey/sa/view',array('surveyId' => $surveyId)),
        );

        $aSettings=array();
        $userIDHelp = "";
        $matomoUserId = $this->get('matomoUserId', 'Survey', $surveyId);
        if(empty($matomoUserId)) {
            $matomoUserId = $this->get('matomoUserId', null, null,'');
        }
        if($matomoUserId) {
            $userIDHelp = sprintf($this->gT("Actual value: %s"),$this->_getHtmlExpression($matomoUserId,$surveyId));
        }
        $aBaseSettings = array(
            'trackSurveyPages' => array(
                'type' => 'select',
                'options' => array(
                    'Y'=> gT('Yes'),
                    'N'=> gT('No'),
                ),
                'label' => $this->gT("Track this survey ."),
                'htmlOptions' => array(
                    'empty' => CHtml::encode(sprintf($this->gT("Use default (%s)"),($this->get('trackSurveyPages',null,null,1) ? gT('Yes') : gT('No')))),
                ),
                'help' => "",
                'current'=>$this->get('trackSurveyPages', 'Survey', $surveyId),
            ),
            'matomoUserId'=>array(
                'type'=>'string',
                'label'=>$this->gT('Set matomo user id'),
                'help' => $userIDHelp,
                'current'=>$this->get('matomoUserId', 'Survey', $surveyId),
            ),
        );
        $aSettings[$this->gT("Basic setting")] = $aBaseSettings;

        $aCustomVariables = array();
        $customVariablesNumber =  $this->get('customVariablesNumber',null,null,0);
        if($customVariablesNumber > 0 ) {
            $customBaseVariables = $this->get('customVariables',null,null,"[]");
            $customBaseVariables = json_decode($customBaseVariables,1);
            $customVariables = $this->get('customVariables', 'Survey', $surveyId,array());
            //~ $customVariablesNumber = isset($pluginSettings['customVariables']) ? intval($pluginSettings['customVariablesNumber']['current']) : 0;
            for ($variableCount = 1; $variableCount <= $customVariablesNumber; $variableCount++) {
                $label = isset($customBaseVariables[$variableCount]['label']) ? $customBaseVariables[$variableCount]['label'] : '';
                if($label) {
                    $current = isset($customVariables[$variableCount]) ? $customVariables[$variableCount] : null;
                    $final = $current;
                    if(empty($final)) {
                        $final = isset($customBaseVariables[$variableCount]['value']) ? $customBaseVariables[$variableCount]['value'] : null;
                    }
                    $help = "";
                    if($final) {
                        $help = sprintf($this->gT("Actual value: %s"),$this->_getHtmlExpression($final,$surveyId));
                    }
                    $aCustomVariables["customVariables[$variableCount]"] = array(
                        'type'=>'string',
                        'label'=>sprintf($this->gT("%s (Custom variable)"),$label),
                        'current'=>$current,
                        'help' => $help,
                    );
                }
            }
            $aSettings[$this->gT("Custom variables")] = $aCustomVariables;
        }

        $aCustomDimensions = array();
        $customDimensionNumber =  $this->get('customDimensionsNumber',null,null,0);
        if($customDimensionNumber > 0 ) {
            $customBaseDimensions = $this->get('customDimensions',null,null,"[]");
            $customBaseDimensions = json_decode($customBaseDimensions,1);
            $customVariables = $this->get('customDimensions', 'Survey', $surveyId,array());
            for ($dimensionsCount = 1; $dimensionsCount <= $customDimensionNumber; $dimensionsCount++) {
                $current = isset($customVariables[$dimensionsCount]) ? $customVariables[$dimensionsCount] : null;
                $final = $current;
                if(empty($final)) {
                    $final = isset($customBaseVariables[$dimensionsCount]['value']) ? $customBaseVariables[$dimensionsCount]['value'] : null;
                }
                $help = "";
                if($final) {
                    $help = sprintf($this->gT("Actual value: %s"),$this->_getHtmlExpression($final,$surveyId));
                }
                $aCustomDimensions["customDimensions[$dimensionsCount]"] = array(
                    'type'=>'string',
                    'label'=>sprintf($this->gT("Dimension #%s"),$dimensionsCount),
                    'current'=>$current,
                    'help' => $help,
                );
            }
            $aSettings[$this->gT("Custom dimensiions")] = $aCustomDimensions;
        }

        $aData['aSettings'] = $aSettings;
        $content = $this->renderPartial('settings', $aData, true);
        return $content;
    }

    /**
     * The main function
     * @param int $surveyId Survey id
     * @return void (redirect)
     */
    public function actionSaveSettings($surveyId)
    {
        if(empty(App()->getRequest()->getPost('save'.get_class($this)))) {
            throw new CHttpException(400);
        }
        $oSurvey=Survey::model()->findByPk($surveyId);
        if(!$oSurvey) {
            throw new CHttpException(404,gT("This survey does not seem to exist."));
        }
        if(!Permission::model()->hasSurveyPermission($surveyId,'surveysettings','update')){
            throw new CHttpException(403);
        }
        /* The settings save */
        \PluginSetting::model()->deleteAll("plugin_id = :pluginid AND model = :model AND model_id = :sid",array(":pluginid"=>$this->id,":model"=>'Survey',':sid'=>$surveyId));
        /* Base settings */
        $this->set('trackSurveyPages', App()->getRequest()->getPost('matomoPublicTrack',""), 'Survey', $surveyId);
        /* Custom variables */
        $customVariables = (array) App()->getRequest()->getPost('customVariables',null);
        $this->set('customVariables', $customVariables, 'Survey', $surveyId);
        /* Custom dimensions */
        $customDimensions = (array) App()->getRequest()->getPost('customDimensions',null);
        $this->set('customDimensions', $customDimensions, 'Survey', $surveyId);
        if(App()->getRequest()->getPost('save'.get_class($this)) == 'redirect') {
            $redirectUrl = Yii::app()->createUrl('admin/survey/sa/view',array('surveyId' => $surveyId));
            Yii::app()->getRequest()->redirect($redirectUrl);
        }
        $redirectUrl = Yii::app()->createUrl('admin/pluginhelper/sa/sidebody',array('plugin' => get_class($this),'method' => 'actionSettings','surveyId' => $surveyId));
        Yii::app()->getRequest()->redirect($redirectUrl,true,303);
    }
    /**
     * @see parent:getPluginSettings
     */
    public function getPluginSettings($getValues=true)
    {
        $pluginSettings= parent::getPluginSettings($getValues);
        if($getValues){
            $customVariables = 0;
            $customVariablesNumber = isset($pluginSettings['customVariablesNumber']['current']) ? intval($pluginSettings['customVariablesNumber']['current']) : 0;
            $customVariables = isset($pluginSettings['customVariables']['current']) ? @json_decode($pluginSettings['customVariables']['current'],1) : null;
            //~ $customVariablesNumber = isset($pluginSettings['customVariables']) ? intval($pluginSettings['customVariablesNumber']['current']) : 0;
            for ($variableCount = 1; $variableCount <= $customVariablesNumber; $variableCount++) {
                $current = array(
                    'label' => isset($customVariables[$variableCount]['label']) ? $customVariables[$variableCount]['label'] : '',
                    'value'=> isset($customVariables[$variableCount]['value']) ? $customVariables[$variableCount]['value'] : '',
                );
                $pluginSettings["customVariables[{$variableCount}][label]"] = array(
                    'type'=>'string',
                    'label'=>sprintf($this->gT("Custom variable label %s"),$variableCount),
                    'current'=>$current['label'],
                    'prefix'=>'customVariables',
                );
                $pluginSettings["customVariables[{$variableCount}][value]"] = array(
                    'type'=>'string',
                    'label'=>sprintf($this->gT("Custom variable %s"),$variableCount),
                    'current'=>$current['value'],
                    'help' => $variableCount ==1 ? $this->gT("You can use expression manager, remind to add { and }") : "",
                );
            }
            $customDimensions = 0;
            $customDimensionsNumber = isset($pluginSettings['customDimensionsNumber']['current']) ? intval($pluginSettings['customDimensionsNumber']['current']) : 0;
            $customDimensions = isset($pluginSettings['customDimensions']['current']) ? @json_decode($pluginSettings['customDimensions']['current'],1) : null;
            //~ $customDimensionsNumber = isset($pluginSettings['customDimensions']) ? intval($pluginSettings['customDimensionsNumber']['current']) : 0;
            for ($dimensionsCount = 1; $dimensionsCount <= $customDimensionsNumber; $dimensionsCount++) {
                $current = array(
                    'value'=> isset($customDimensions[$dimensionsCount]) ? $customDimensions[$dimensionsCount] : '',
                );
                $pluginSettings["customDimensions[{$dimensionsCount}]"] = array(
                    'type'=>'string',
                    'label'=>sprintf($this->gT("Custom dimension id %s value"),$dimensionsCount),
                    'current'=>$current['value'],
                    'help' => $dimensionsCount ==1 ? $this->gT("You can use expression manager, remind to add { and }") : "",
                );
            }
        }
        unset($pluginSettings['customVariables']);
        unset($pluginSettings['customDimensions']);
        return $pluginSettings;
    }

    /**
    * set actual url when activate
    * @see parent::saveSettings()
    */
    public function saveSettings($settings)
    {
        if(!empty($settings['customVariablesNumber']))
        {
            $customVariables = Yii::app()->getRequest()->getPost('customVariables');
            $settings['customVariables'] = json_encode($customVariables);
        }
        if(!empty($settings['customDimensionsNumber']))
        {
            $customDimensions = Yii::app()->getRequest()->getPost('customDimensions');
            $settings['customDimensions'] = json_encode($customDimensions);
        }
        parent::saveSettings($settings);
    }

    /** Add the setting specific to survey **/
    public function beforeSurveySettings()
    {
    }

    /* Basic params */
    public function beforeSurveyPage() {
        $this->aMatomoCustomUrlParams['base'] = 'survey';
        $this->aMatomoCustomUrlParams['sid'] = $this->getEvent()->get('surveyId');
        $this->aMatomoCustomUrlParams['lang'] = Yii::app()->getLanguage();
        $surveyId = $this->getEvent()->get('surveyId');

        $this->_setCustomURL($surveyId);
        if(in_array(Yii::app()->request->getParam('action'),array('previewgroup', 'previewquestion'))) {
            $this->aMatomoCustomUrlParams['event'] = Yii::app()->getRequest()->get('action');
        }
        if (Yii::app()->request->getParam('loadall') == "reload") {
            $this->aMatomoCustomUrlParams['event'] = Yii::app()->request->getParam('loadall');
        }
        if (Yii::app()->request->getParam('clearall') == "clearall") {
            $this->aMatomoCustomUrlParams['event'] = Yii::app()->request->getParam('clearall');
        }
        if (Yii::app()->request->getParam('saveall') == "saveall") {
            $this->aMatomoCustomUrlParams['event'] = Yii::app()->request->getParam('saveall');
        }
    }

    /* Question params */
    public function beforeQuestionRender() {
        $oSurvey = Survey::model()->findByPk($this->getEvent()->get('sid'));
        $this->aMatomoCustomUrlParams['gid'] = $this->getEvent()->get('gid');
        $this->aMatomoCustomUrlParams['qid'] = $this->getEvent()->get('qid');
    }

    /* Event params */
    public function afterSurveyComplete() {
        $this->aMatomoCustomUrlParams['event'] = 'complete';
    }

    /* Event params */
    public function afterSurveyQuota() {
        $this->aMatomoCustomUrlParams['event'] = 'quota';
        // TODO : add quota id
    }

    /* To have controller */
    public function beforeControllerAction() {
        if($this->getEvent()->get('controller') == 'admin') {
            return;
        }
        // plugins ?
        $this->sMatomoController = $this->getEvent()->get('controller');
        /* TODO switch controller */
        /* TODO action */
    }

    /* Add the script and the image */
    public function beforeCloseHtml() {
        if($this->get('respectDNT')) {
            $bDoNotTrackEnable = false;
            if (array_key_exists('HTTP_DNT', $_SERVER) && (1 === (int) $_SERVER['HTTP_DNT'])) {
                $bDoNotTrackEnable = true;
            }
            if($bDoNotTrackEnable) {
                return;
            }
        }
        //~ $this->aMatomoController = $this->getEvent()->get('controller');
        //$sessionSurvey=Yii::app()->session["survey_{$surveyId}"];
        $surveyId = $this->getEvent()->get('surveyId');
        $trackSurveyPages = $this->_getCurrentSetting('trackSurveyPages',$surveyId);

        if(!$trackSurveyPages) {
            return;
        }
        $matomoURL = $this->_getCurrentSetting('matomoURL',$surveyId); // Encode ???
        $siteID = intval($this->_getCurrentSetting('siteID',$surveyId));
        $matomoScriptInit = "var _paq = window._paq || [];";
        App()->getClientScript()->registerScript('matomoScriptInit',$matomoScriptInit,CClientScript::POS_BEGIN);
        $this->_setCustomURL($surveyId);
        $this->_setCustomVariable($surveyId);
        $this->_setCustomDimension($surveyId);
        $this->_setCustomUserId($surveyId);
        $matomoScriptEnd = "_paq.push(['trackPageView']);\n"
             . "_paq.push(['enableLinkTracking']);\n"
             . "(function() {\n"
             . "  var u=\"{$matomoURL}\";\n"
             . "  _paq.push(['setTrackerUrl', u+'matomo.php']);\n"
             . "  _paq.push(['setSiteId', '{$siteID}']);\n"
             . "  var d=document, g=d.createElement('script'), s=d.getElementsByTagName('script')[0];\n"
             . "  g.type='text/javascript'; g.async=true; g.defer=true; g.src=u+'matomo.js'; s.parentNode.insertBefore(g,s);\n"
             . "})();\n";
        App()->getClientScript()->registerScript('matomoScriptEnd',$matomoScriptEnd,CClientScript::POS_BEGIN);
        //~ $html = "<noscript><p><img src='{$matomoURL}/matomo.php?idsite={$siteID}&amp;rec=1' style='border:0;' alt='' /></p></noscript>\n";
        //~ $html .="<pre>".print_r($this->aMatomoCustomUrlParams,1)."</pre>";
        //~ $this->getEvent()->set('html', $html);

    }

    /**
     * Add the setCustomVariable to js
     * @param integer $surveyId
     * @return $void
     */
    private function _setCustomVariable($surveyId){
        $aMatomoCustomVariables = array();
        $customVariablesNumber =  $this->get('customVariablesNumber',null,null,0);
        if($customVariablesNumber > 0 ) {
            $aSurveyCustomVariables = $this->get("customVariables",'survey',$surveyId);
            $aGlobalCustomVariables = json_decode($this->get("customVariables",null,null),1);
            for ($variablesCount = 1; $variablesCount <= $customVariablesNumber; $variablesCount++) {
                if(!empty($aGlobalCustomVariables[$variablesCount]['label'])) {
                    $label = $aGlobalCustomVariables[$variablesCount]['label'];
                    $currentVariableValue = !empty($aSurveyCustomVariables[$variablesCount]) ? $aSurveyCustomVariables[$variablesCount] : null;
                    if(empty($currentVariableValue)) {
                        $currentVariableValue = !empty($aGlobalCustomVariables[$variablesCount]['value']) ? $aGlobalCustomVariables[$variablesCount]['value'] : null;
                    }
                    if(!empty($currentVariableValue)) {
                        $currentVariableValue = trim(LimeExpressionManager::ProcessStepString($currentVariableValue,array(),3,true));// force static
                        if($currentVariableValue) {
                            $aMatomoCustomVariables[] = "_paq.push(['setCustomVariable',{$variablesCount},'".CHtml::encode($label)."','".CHtml::encode($currentVariableValue)."','visit']);";
                        }
                    }
                }
            }
        }
        if(empty($aMatomoCustomVariables)) {
            return;
        }
        $matomoCustomVariables = implode("\n",$aMatomoCustomVariables);
        App()->getClientScript()->registerScript('matomoCustomVariables',$matomoCustomVariables,CClientScript::POS_BEGIN);
    }

    /**
     * Add the setCustomDimension to js
     * @param integer $surveyId
     * @return $void
     */
    private function _setCustomDimension($surveyId){
        $aMatomoCustomDimensions = array();
        $customDimensionNumber =  $this->get('customDimensionsNumber',null,null,0);
        if($customDimensionNumber > 0 ) {
            $aSurveyCustomDimensions = $this->get("customDimensions",'survey',$surveyId);
            $aGlobalCustomDimensions = json_decode($this->get("customDimensions",null,null),1);
            for ($dimensionsCount = 1; $dimensionsCount <= $customDimensionNumber; $dimensionsCount++) {
                $currentDimensionValue = !empty($aSurveyCustomDimensions[$dimensionsCount]) ? $aSurveyCustomDimensions[$dimensionsCount] : null;
                if(empty($currentDimensionValue)) {
                    $currentDimensionValue = !empty($aGlobalCustomDimensions[$dimensionsCount]) ? $aGlobalCustomDimensions[$dimensionsCount] : null;
                }
                if(!empty($currentDimensionValue)) {
                    $currentDimensionValue = trim(LimeExpressionManager::ProcessStepString($currentDimensionValue,array(),3,true));// force static
                    if($currentDimensionValue) {
                        $aMatomoCustomDimensions[] = "_paq.push(['setCustomDimension',{$dimensionsCount},'".CHtml::encode($currentDimensionValue)."']);";
                    }
                }
            }
        }
        if(empty($aMatomoCustomDimensions)) {
            return;
        }
        $matomoCustomDimensions = implode("\n",$aMatomoCustomDimensions);
        App()->getClientScript()->registerScript('matomoCustomDimensions',$matomoCustomDimensions,CClientScript::POS_BEGIN);
    }

    /**
     * Add the setCustomDimension to js
     * @param integer $surveyId
     * @return $void
     */
    private function _setCustomUserId($surveyId)
    {
        $sCustomUserid = $this->_getCurrentSetting('matomoUserId',$surveyId);
        if(empty($sCustomUserid)) {
            return;
        }
        $sCustomUserid = trim(LimeExpressionManager::ProcessStepString($sCustomUserid,array(),3,true));// force static
        if(empty($sCustomUserid)) {
            return;
        }
        $matomoCustomUserId = "_paq.push(['setUserId', '".CHtml::encode($sCustomUserid)."']);";
        App()->getClientScript()->registerScript('matomoCustomUserId',$matomoCustomUserId,CClientScript::POS_BEGIN);
    }

    /**
     * Add the setCustomUrl to js
     * @param integer $surveyId
     * @param string[] $aUrlInfo forced params
     * @return $void
     */
    private function _setCustomURL($surveyId,$aUrlInfo = array()){
        $rewriteURLs =$this->_getCurrentSetting('rewriteURLs', $surveyId);
        if ($rewriteURLs) {
            //Write the custom URL to the piwikForLimeSurvey JS variable. This is then read inside loadPiwikTrackingCode()
            $this->aMatomoCustomUrlParams=array_replace($this->aMatomoCustomUrlParams,$aUrlInfo);
            switch($rewriteURLs) {
                case 'admin':
                    $sCustomUrl=$this->_getAdminUrl();
                    break;
                case 'public':
                    $sCustomUrl=$this->_getPublicUrl();
                    break;
                case 'tweak':
                    $sCustomUrl=$this->_getTweakUrl();
                    break;
                default:
                    $sCustomUrl=null;
                    break;
            }
            tracevar($sCustomUrl);
            if($sCustomUrl) {
                $matomoCustomUrl="_paq.push(['setCustomUrl', '$sCustomUrl']);";
                App()->getClientScript()->registerScript('matomoCustomUrl',$matomoCustomUrl,CClientScript::POS_BEGIN);
            }
        }
    }

    /**
     * A function to rewrite to admin page with actual information
     * Return string
     */
    private function _getAdminUrl()
    {
        $aParams=$this->aMatomoCustomUrlParams;
        $sController='admin/survey/';
        switch ($aParams['base']) // Todo : other controller
        {
            case 'survey':
                $aParams['sa']='view';
                break;
            default:
                $aParams['sa']='index';
                $aParams['event']=($aParams['event']) ? $aParams['event'] : $aParams['base'];
                // leave base if other than survey listing
                break;
        }
        unset($aParams['base']);
        tracevar($aParams);
        $sController.=Yii::app()->getUrlManager()->createPathInfo(array_filter($aParams),"/","/");
        return Yii::app()->createUrl($sController);
    }

    /**
     * A function to rewrite to publi page with actual information
     * Return string
     */
    private function _getPublicUrl()
    {
        $aParams=$this->aMatomoCustomUrlParams;
        switch ($aParams['base']) // Todo : other controller
        {
            case 'survey':
                $sController='survey/index/';
                break;
            case 'statistics_user':
                $sController='statistics_user/view/'; // Not sure for this one
                break;
            default:
                $sController='surveys/index/';
                $aParams['event']=$aParams['base'];
                break;
        }
        $sController.=Yii::app()->getUrlManager()->createPathInfo(array_filter($aParams),"/","/");
        return Yii::app()->createUrl($sController);
    }

    /**
     * A function to rewrite to a false url but with cleaner view in matomo
     * Return string
     */
    private function _getTweakUrl()
    {
        $aParams=$this->aMatomoCustomUrlParams;
        unset($aParams['base']);
        $sTweakedUrl = Yii::app()->getUrlManager()->createPathInfo(array_filter($aParams),"-","/");
        $sTweakedUrl = str_replace("sid-","",$sTweakedUrl);
        $sTweakedUrl = str_replace("lang-","",$sTweakedUrl);
        return $sTweakedUrl;
    }

    /**
     * Get current setting for current survey (use empty string as null value)
     * @param string setting to get
     * @param integer survey id
     * @return string|array|null
     */
    private function _getCurrentSetting($setting,$surveyId = null)
    {
        if($surveyId) {
            $value = $this->get($setting,'Survey',$surveyId,'');
            if($value !== '') {
                return $value;
            }
        }
        $default = (isset($this->settings[$setting]['default'])) ? $this->settings[$setting]['default'] : null;
        return $this->get($setting,null,null,$default);
    }

    /**
     * Translation for plugin
     *
     * @param string $sToTranslate The message that are being translated
     * @param string $sEscapeMode
     * @param string $sLanguage
     * @return string
     */
    public function gT($sToTranslate, $sEscapeMode = 'unescaped', $sLanguage = null)
    {
        return parent::gT($sToTranslate, $sEscapeMode, $sLanguage);
    }

  /**
  * Get the complete HTML from a string with Expression for admin
  * @param string $sExpression : the string to parse
  * @param array $aReplacement : optionnal array of replacemement
  * @param boolean $forceEm : force EM or not
  *
  * @author Denis Chenu
  * @version 1.0
  */
  private function _getHtmlExpression($sExpression,$iSurveyId,$aReplacement=array(),$forceEm=false)
  {
      $LEM = LimeExpressionManager::singleton();
      $aReData=array();
      if($iSurveyId) {
          $LEM::StartSurvey($iSurveyId, 'survey', array('hyperlinkSyntaxHighlighting'=>true));// replace QCODE
      }
      if($forceEm) {
          $sExpression = "{".$sExpression."}";
      }else{
          $oFilter = new CHtmlPurifier();
          $sExpression =$oFilter->purify(viewHelper::filterScript($sExpression));
      }
      LimeExpressionManager::ProcessString($sExpression);
      //~ templatereplace($sExpression, $aReplacement,$aReData,__CLASS__,false,null,array(),true);
      return $LEM::GetLastPrettyPrintExpression();
  }
}
